package tsp.algorithm;

import java.math.BigInteger;

import tsp.init.StartProblem;
import tsp.model.Permutation;

public abstract class Algorithm {

	public static final String ALGORITHM_TYPE_ERROR_MESSAGE = 
		"Parametr '"+StartProblem.AlGORITHM_NAME_PARAMETER+"' moze przyjmowac wartosci ze zbioru: " +
			"{'H','G','S','R'}.\n" +
			"Odpowiadaja one algorytmom: simpleHeuristics, Greedy, Steepest, Random.";
	public static char TYPE_GREEDY_NAME = 'G';
	public static char TYPE_STEEPEST_NAME = 'S';
	public static char TYPE_RANDOM_NAME = 'R';
	public static char TYPE_SIMPLE_HEURISTICS_NAME = 'H';
	public static char TYPE_TABU_SEARCH_NAME = 'T';
	public static char TYPE_SIMULATED_ANNEALING_NAME = 'W';
	
	protected Permutation beginPermutation;
	protected Permutation endPermutation;
	private int maxWorkMiliseconds = 0;
	private BigInteger maxSteps = BigInteger.ZERO;
	private BigInteger stepsMade = BigInteger.ZERO;
	private BigInteger nodesChecked = BigInteger.ZERO; 
	private long startTime;
	
	public Algorithm(Permutation permutation) {
		beginPermutation = permutation;
	}

	public abstract Permutation findSolution(Permutation p);

	public abstract Permutation findSolution();
	
	public abstract String getName();

	public void setBeginPermutation(Permutation p) {
		beginPermutation = p;
	}
	
	public Permutation getBeginPermutation() {
		return beginPermutation;
	}

	public Permutation getEndPermutation() {
		return endPermutation;
	}
	
	public void goToNode(Permutation permutation, int k, int l){
		stepsMade = stepsMade.add(BigInteger.ONE);
		permutation.swap(k, l);
	}
	
	public Double checkCitiesEvaluation(Permutation permutation, int k, int l){
		nodesChecked = nodesChecked.add(BigInteger.ONE);
		return permutation.swapCitiesEvaluation(k, l);
	}
	
	protected void initAlgorithm(){
		beginPermutation.countEvaluation();
		nodesChecked = BigInteger.ZERO;
		stepsMade = BigInteger.ZERO;
		startTime = System.currentTimeMillis();
	}
	
	protected boolean isTimeToFinish(){
		if((maxWorkMiliseconds != 0 && 
				System.currentTimeMillis() - startTime > maxWorkMiliseconds) ||
				(!maxSteps.equals(BigInteger.ZERO) && stepsMade.compareTo(maxSteps) >= 0)){
			return true;
		}else{
			return false;
		}
	}

	public int getMaxWorkMiliseconds() {
		return maxWorkMiliseconds;
	}

	public void setMaxWorkMiliseconds(int maxWorkMiliseconds) {
		this.maxWorkMiliseconds = maxWorkMiliseconds;
	}

	public BigInteger getMaxSteps() {
		return maxSteps;
	}

	public void setMaxSteps(BigInteger maxSteps) {
		this.maxSteps = maxSteps;
	}

	public BigInteger getStepsMade() {
		return stepsMade;
	}

	public BigInteger getNodesChecked() {
		return nodesChecked;
	}
	
	public abstract char getAlgorithmName();
}
