package tsp.algorithm;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import tsp.model.Permutation;

public class SimulatedAnnealingAlgorithm extends Algorithm {
	int notBetterPermutationsNumber = 0;
	int maxNotBetterPermutationsNumber = 0;
	Double cknum;//parameter Ck
	int knum = 0;//number of iteration of parameter Ck
	private Double alpha;

	public SimulatedAnnealingAlgorithm(Permutation permutation) {
		super(permutation.cloning());
	}

	@Override
	public Permutation findSolution(Permutation p) {
		beginPermutation = p;
		return findSolution();
	}

	@Override
	public Permutation findSolution() {
		Double nextEval = 0.0d;
		Double lastEval = 0.0d; 
		Permutation permutation = beginPermutation.cloning();
		
		SimpleHeuristics shSupportAlgorithm = new SimpleHeuristics(permutation.cloning());
		permutation = shSupportAlgorithm.findSolution();
		
		endPermutation = permutation.cloning();
		nextEval = endPermutation.countEvaluation();
		initAlgorithm();
		knum = 0;
		//0.0000001;//
		cknum = calculateCknum(0.95d, 1000, permutation.cloning());//trzeba wyliczyć: powinno być takie, że warunek 2 w stepCondition() powinien obejmować 95 % przypadków 
		alpha = 0.99999d;
		notBetterPermutationsNumber = 0;
		maxNotBetterPermutationsNumber = (int) Math.pow(beginPermutation.getLength(), 2);
		int k = 0, l = 1;
		Random generator = new Random(System.nanoTime());
		while(!stopCondition(lastEval, nextEval)){
			k = 0;
			l = 1;
			lastEval = permutation.countEvaluation();
			permutation.setActualIndex(generator.nextInt(permutation.getLength()));
			while(k < permutation.getLength()-1){
				nextEval = checkCitiesEvaluation(permutation, k, l);
				if(stepCondition(lastEval, nextEval)){
					if(permutation.countEvaluation() < endPermutation.getLastEvaluation()){
						endPermutation = permutation.cloning();
					}
					goToNode(permutation, k, l);
					break;
				}
				l++;
				if(l == permutation.getLength()){
					k++;
					l = k+1;
				}			
			}
			if(k >= permutation.getLength()-1){
				break;
			}
			knum++;
			cknum = alpha * cknum;
		}
		return endPermutation;
	}

	private Double calculateCknum(Double allowingPercent, int probeSize, Permutation beginPermutation) {
		List<Double> resultsList = new ArrayList<Double>();
		double lastEval = beginPermutation.countEvaluation();
		int counter = 0;
		while(true){
			for(int i = 0; i < beginPermutation.getLength(); i++){
				for(int j = i+1; j < beginPermutation.getLength(); j++){					
					resultsList.add(beginPermutation.swapCitiesEvaluation(i, j) - lastEval);
					counter++;
					if(counter >= probeSize){
						break;
					}
				}
				if(counter >= probeSize){
					break;
				}
			}
			if(counter >= probeSize){
				break;
			}
			lastEval = beginPermutation.randomSwapEvaluation();
		}
		Collections.sort(resultsList);
		return - resultsList.get((int) (probeSize * allowingPercent)) / Math.log(allowingPercent);//0.5);//
	}

	private boolean stepCondition(Double lastEval, Double nextEval) {
		if(lastEval.compareTo(nextEval) >= 0){
			return true;
		}
		if(Math.exp(-(nextEval - lastEval)/cknum) > Math.random()){
			return true;
		}
		return false;
	}

	//should be 10*liczba_sasiadow - max number of not better permutations
	private boolean stopCondition(Double lastEval, Double nextEval) {
		if(lastEval.compareTo(nextEval) >= 0){//brak polepszenia
			notBetterPermutationsNumber++;
//			System.out.println(notBetterPermutationsNumber+"/"+maxNotBetterPermutationsNumber);
		}else{//jest polepszenie!
			notBetterPermutationsNumber = 0;
		}
		if(notBetterPermutationsNumber >= maxNotBetterPermutationsNumber){
			return true;
		}else{
//			return false;
			return isTimeToFinish();
		}
//		if(getStepsMade().mod(BigInteger.valueOf(128)).intValue() == 0)
//		else
//			return false;
	}

	@Override
	public String getName() {
		return "SimulatedAnnealing";
	}

	@Override
	public char getAlgorithmName() {
		return Algorithm.TYPE_SIMULATED_ANNEALING_NAME;
	}

}
