package tsp.algorithm;

import tsp.model.Permutation;

public class SteepestAlgorithm extends Algorithm {

	public SteepestAlgorithm(Permutation permutation) {
		super(permutation.cloning());
	}

	@Override
	public Permutation findSolution(Permutation p) {
		beginPermutation = p;
		return findSolution();
	}

	@Override
	public Permutation findSolution() {
		initAlgorithm();
		Permutation permutation = beginPermutation.cloning();
		Double actualEvaluation = permutation.countEvaluation();
		Double gainLimit = 0.0d;
		int k = 0, l = 1, bestMoveK = -1, bestMoveL = -1;
		Double bestEvaluation = actualEvaluation - gainLimit;
		Double tempEvaluation = 0.0d;
		int result = 1;
		while(result > 0){
			actualEvaluation = bestEvaluation;
			bestMoveK = -1;
			bestMoveL = -1;
			k = 0;
			l = 1;
			while(k < permutation.getLength()-1){ 
				tempEvaluation = checkCitiesEvaluation(permutation, k, l);
				if(bestEvaluation.compareTo(tempEvaluation) > 0){
					bestEvaluation = tempEvaluation;
					bestMoveK = k;
					bestMoveL = l;
				}
				l++;
				if(l == permutation.getLength()){
					k++;
					l = k+1;
				}			
			}
			if(bestMoveK != -1){
				goToNode(permutation, bestMoveK, bestMoveL);
			}
			if(isTimeToFinish()){
				break;
			}
			result = ((Double)(actualEvaluation - bestEvaluation)).compareTo(gainLimit);
		}
		return endPermutation = permutation;
	}

	@Override
	public String getName() {
		return "Steepest";
	}

	@Override
	public char getAlgorithmName() {
		return Algorithm.TYPE_STEEPEST_NAME;
	}

}
