package tsp.algorithm;

import java.util.ArrayList;
import java.util.List;

import tsp.algorithm.utils.MastersList;
import tsp.algorithm.utils.NeighbourMove;
import tsp.algorithm.utils.TabuList;
import tsp.model.Permutation;

public class TabuSearchAlgorithm extends Algorithm {
	
	private Double tabuLengthFactor = 0.5d;
	private Double mastersLengthFactor = 0.5d;
	int notBetterPermutationsNumber = 0;
	int maxNotBetterPermutationsNumber = 0;

	public TabuSearchAlgorithm(Permutation permutation) {
		super(permutation.cloning());
	}

	@Override
	public Permutation findSolution(Permutation p) {
		beginPermutation = p;
		return findSolution();
	}

	@Override
	public Permutation findSolution() {
		Double nextEval = 0.0d;
		Double lastEval = 0.0d; 
		Permutation permutation = beginPermutation.cloning();
		
		SimpleHeuristics shSupportAlgorithm = new SimpleHeuristics(permutation.cloning());
		permutation = shSupportAlgorithm.findSolution();
		
		endPermutation = permutation.cloning();
		nextEval = endPermutation.countEvaluation();
		initAlgorithm();
		notBetterPermutationsNumber = 0;
		maxNotBetterPermutationsNumber = beginPermutation.getLength() * 10;
		
		List<NeighbourMove> candidates;
		TabuList tabu = new TabuList((int) Math.round((beginPermutation.getLength())*tabuLengthFactor), beginPermutation.getLength());
		long size = Math.round((beginPermutation.getLength())*mastersLengthFactor);
		if(size > Integer.MAX_VALUE){
			throw new Error("TabuSearch: Too long permutation!");
		}
		MastersList masters = new MastersList((int) size);
		while(!stopCondition(lastEval, nextEval)){
			if(!masters.canGetNext(permutation, this)){
				candidates = getCandidates(tabu, permutation);				
				masters.setNewList(candidates, permutation);
			}
			goToNode(permutation, masters.getNextNeighbour(), tabu);
			lastEval = nextEval;
			nextEval = permutation.getLastEvaluation();
		}
		return endPermutation;
	}

	private List<NeighbourMove> getCandidates(TabuList tabu,
			Permutation permutation) {
		List<NeighbourMove> result = new ArrayList<NeighbourMove>();
		for(int i = 0; i < permutation.getLength(); i++){
			for(int j = i+1; j < permutation.getLength(); j++){
				if(!tabu.isTabu(i,j)){
					addToList(result, new NeighbourMove(i, j, checkCitiesEvaluation(permutation, i, j)));
				}
			}
		}
		return result;
	}

	private void addToList(List<NeighbourMove> result,
			NeighbourMove neighbourMove) {
		int i = 0;
		while(i < result.size() && result.get(i).getEval() < neighbourMove.getEval()){
			i++;
		}
		result.add(i, neighbourMove);
	}

	private void goToNode(Permutation permutation, NeighbourMove move, TabuList tabu) {
		tabu.updateTabu(move);
		goToNode(permutation, move.getOne(), move.getTwo());
		if(permutation.countEvaluation() < endPermutation.getLastEvaluation()){
			endPermutation = permutation.cloning();
		}
	}

	//should be 10*liczba_sasiadow - max number of not better permutations
	private boolean stopCondition(Double lastEval, Double nextEval) {
		if(lastEval.compareTo(nextEval) >= 0){//brak polepszenia
			notBetterPermutationsNumber++;
//			System.out.println(notBetterPermutationsNumber+"/"+maxNotBetterPermutationsNumber);
//		}else{//jest polepszenie!
//			notBetterPermutationsNumber = 0;
		}
		if(notBetterPermutationsNumber >= maxNotBetterPermutationsNumber){
			return true;
		}else{
//			return false;
			return isTimeToFinish();
		}
//		if(getStepsMade().mod(BigInteger.valueOf(128)).intValue() == 0)
//		else
//			return false;
	}

	@Override
	public String getName() {
		return "TabuSearch";
	}

	@Override
	public char getAlgorithmName() {
		return Algorithm.TYPE_TABU_SEARCH_NAME;
	}

}
