package tsp.init;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import tsp.algorithm.Algorithm;

public class MultiTSPSolver {

	/**
	 * @param args:
	 * args[0]: int multi-random-times, 
	 * args[1]: String outputFileName,
	 * args[2]: boolean showSolution: 0/1,
	 * args[3]: boolean saveSolutionToImage: 0/1,
	 * args[4]: [String spacer]
	 * args[5]: [String algorithmName]
	 */
	public static void main(String[] args) {
		if (args == null || args.length < 4) {
			System.out.println("Podaj wszystkie potrzebne argumenty! '"+StartProblem.LAUCHING_TIMES_NR_PARAMETER+" "+
					StartProblem.OUTPUT_FILE_PARAMETER+" "+StartProblem.SHOW_SOLUTION_PARAMETER+" "+StartProblem.SAVE_PICTURE_PARAMETER+
					" ["+StartProblem.OUTPUT_SPACER_PARAMETER+" ["+StartProblem.AlGORITHM_NAME_PARAMETER+"]]'");
			System.out.println("Przykładowo: C:\\java -jar thisJar.jar 100 sh_test.txt 1 1 ; "+
					Algorithm.TYPE_SIMPLE_HEURISTICS_NAME);
			System.out.println("Spowoduje wykonanie 100 razy algorytmu SimpleHeuristics dla wszystkich problemów " +
					"(pliki z rozszerzeniem .tsp) jakie znajdują się w katalogu w którym jest niniejszy jar " +
					"i zapisanie wyników, oddzielonych nakiem ';', do pliku 'sh_test.txt', " +
					"oraz po obliczeniach wyświetli okno z widokiem ostatniego rozwiązania, " +
					"jak również zapisze widok tego okna do pliku jpg.");
			System.out.println("Domyślnie wyniki są zapisywane do pliku oddzielone spacją: ' ', " +
					"którą można wybrać jako parametr "+StartProblem.OUTPUT_SPACER_PARAMETER+" przy pomocy znaku 'S'");
			System.out.println(Algorithm.ALGORITHM_TYPE_ERROR_MESSAGE);
			System.out.println("Jeśli nie podasz ostatniego parametru - zostaną wykonane wszystkie algorytmy.");
			System.out.println("Pliki .tsp powinny mieć w nazwie liczbę będącą rozmiarem instancji\n" +
					"np. plik tsp225.tsp powinien zawierać problem TSP o rozmiarze 225. " +
					"W przeciwnym razie wyniki dla kolejnych instancji będą źle posortowane.");
			return;
		}
		if(args.length > 6){
			System.out.println("Za dużo argumentów("+args.length+")! Podaj tylko wymagane argumenty(6)!");
			return;			
		}
		File dir = null;
		try {
			dir = new File(new File(".").getCanonicalPath());
		} catch (IOException e) {
			e.printStackTrace();
		}
		String[] names = dir.list();
		String[] argsa = new String[args.length+1];
		for(int i = 1; i < argsa.length; i++){
			argsa[i] = args[i-1];
		}
		String folderName = "MultiTSP_"+System.currentTimeMillis()+"";
		List<String> problemNames = new ArrayList<String>();
		List<Integer> problemSizes = new ArrayList<Integer>();
		System.out.println("Znalezione pliki:");
		for (int i = 0; i < names.length; i++) {
			if(names[i].length() >= 4 && names[i].substring(names[i].length() - 4).equalsIgnoreCase(".tsp")){
				//powyższy warunek przefiltrowuje(przepuszcza dalej) wszystkie pliki z rozszerzeniem .tsp
				System.out.println(names[i]);
				problemNames.add(names[i]);
				problemSizes.add(getSizeFromFilename(names[i]));
			}
		}
		Integer tempInt;
		String tempString;
		for(int i = 0; i < problemSizes.size()-1; i++){
			for(int j = i + 1; j < problemSizes.size(); j++){
				if(problemSizes.get(i) > problemSizes.get(j)){//swap
					tempInt = problemSizes.get(i);
					tempString = problemNames.get(i);
					
					problemSizes.set(i, problemSizes.get(j));
					problemNames.set(i, problemNames.get(j));
					
					problemSizes.set(j, tempInt);
					problemNames.set(j, tempString);
				}
			}			
		}
		System.out.println("Zaczynamy Testowanie! Trzymaj się, żebyś nie spadł z krzesła");
		long time;
		for(String problemName : problemNames){			
			System.out.println("Uruchamiamy '"+problemName+"' ...");
			argsa[0] = problemName; 
			StartProblem.folderName = folderName;
			StartProblem.multiTSPTribe = true;
			time = System.currentTimeMillis();
			StartProblem.main(argsa);
			time = System.currentTimeMillis() - time;
			System.out.println("Wykonanie '"+problemName+"' zajęło: "+time/1000+" sekund");
		}
	}

	private static Integer getSizeFromFilename(String string) {
		String number = "";
		for(int i = 0; i < string.length(); i++){
			if(Character.isDigit(string.charAt(i))){
				number += string.charAt(i);
			}
		}
		return Integer.valueOf(number);
	}

}
