package tsp.init;

import java.awt.EventQueue;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigInteger;

import javax.swing.JComponent;

import tsp.algorithm.Algorithm;
import tsp.algorithm.GreedyAlgorithm;
import tsp.algorithm.RandomAlgorithm;
import tsp.algorithm.SimpleHeuristics;
import tsp.algorithm.SimulatedAnnealingAlgorithm;
import tsp.algorithm.SteepestAlgorithm;
import tsp.algorithm.TabuSearchAlgorithm;
import tsp.model.Permutation;
import tsp.model.Problem;
import tsp.readInput.ProblemReader;
import tsp.readInput.exception.WrongProblemDefinisionException;
import tsp.utils.AlgorithmResultGatherer;
import tsp.utils.CreateImage;
import tsp.utils.TimeCounter;
import tsp.window.TravelPathWindow;

public class StartProblem extends JComponent{

	private static final long serialVersionUID = -2441454769373691330L;
	public static final String AlGORITHM_NAME_PARAMETER = "nazwa_algorytmu";
	public static final String PATH_PARAMETER = "sciezka_do_pliku_z_problemem";
	public static final String OUTPUT_FILE_PARAMETER = "nazw_pliku_wyjsciowego";
	public static final String LAUCHING_TIMES_NR_PARAMETER = "liczba_powtorzen";
	public static final String OUTPUT_SPACER_PARAMETER = "znak_oddzielający_wyniki_w_pliku";
	public static final String SHOW_SOLUTION_PARAMETER = "czy_pokazac_rozw";
	public static final String SAVE_PICTURE_PARAMETER = "zapisac_obraz_rozw";
	public static String folderName = null;
	public static boolean multiTSPTribe = false;
	private static char spacer = ' ';
	static boolean showWindow = false;
	static boolean savePicture = false;

	/**
	 * @param args:
	 * args[0]: String problemFile, 
	 * args[1]: int multi-random-times, 
	 * args[2]: String outputFileName,
	 * args[3]: boolean showSolution: 0/1,
	 * args[4]: boolean saveSolutionToImage: 0/1,
	 * args[5]: [String spacer]
	 * args[6]: [String algorithmName]
	 */
	public static void main(String[] args) {	
		boolean append = false;
		if(args == null || args.length < 5){
			System.out.println("Podaj wszystkie potrzebne argumenty! " +
					"'"+PATH_PARAMETER+" "+LAUCHING_TIMES_NR_PARAMETER+" "+
					OUTPUT_FILE_PARAMETER+" "+SHOW_SOLUTION_PARAMETER+" "+SAVE_PICTURE_PARAMETER+
					" ["+OUTPUT_SPACER_PARAMETER+" ["+AlGORITHM_NAME_PARAMETER+"]]'");
			System.out.println("Przykładowo: C:\\java -jar thisJar.jar tsp255.tsp 100 sh_test.txt 1 1 ; "+
					Algorithm.TYPE_SIMPLE_HEURISTICS_NAME);
			System.out.println("Spowoduje wykonanie 100 razy algorytmu SimpleHeuristics dla problemu " +
					"'tsp255.tsp' i zapisanie wyników, oddzielonych nakiem ';', do pliku 'sh_test.txt', " +
					"oraz po obliczeniach wyświetli okno z widokiem ostatniego rozwiązania, " +
					"jak również zapisze widok tego okna do pliku jpg.");
			System.out.println("Domyślnie wyniki są zapisywane do pliku oddzielone spacją: ' ', " +
					"którą można wybrać jako parametr "+OUTPUT_SPACER_PARAMETER+" przy pomocy znaku 'S'");
			System.out.println(Algorithm.ALGORITHM_TYPE_ERROR_MESSAGE);
			System.out.println("Jeśli nie podasz ostatniego parametru - zostaną wykonane wszystkie algorytmy.");
			return;
		}
		if(args.length > 7){
			System.out.println("Za dużo argumentów("+args.length+")! Podaj tylko wymagane argumenty(7)!");
			return;			
		}
		if(!(new File(args[0])).exists()){
			System.out.println("There's no such problem file in the filesystem. File could not be found!");
			return;
		}
		if(!(new File(args[2])).exists()){
			System.out.println("There's no such ouput file in the filesystem. File will be created!");
		}else{
			System.out.println("Output file already exist in the filesystem. File will be append!");
			append = true;
		}
		if(args.length >= 6){
			if(args[5].charAt(0) != 'S')
				spacer = args[5].charAt(0); 
		}
		if(args[3].charAt(0) == '1')
			showWindow = true;
		if(args[4].charAt(0) == '1')
			savePicture = true;
		
		ProblemReader problemReader = new ProblemReader();
		Problem problem = null;
		try {
			problem = problemReader.read(args[0]);
		} catch (WrongProblemDefinisionException e) {
			System.out.println(e.getMessage());
			return;
		}
		Permutation permutation = problem.generateRandomPermutation();
		Integer multiRandomTimes = Integer.parseInt(args[1]);
		long measuredTime = 0;
		String problemName = getProblemFileName(args[0]);
		TimeCounter t = new TimeCounter();
		if(!multiTSPTribe){//tzn, że StartProblem jest uruchamiany NIE przez MultiTSPSolver
			folderName = System.currentTimeMillis()+"";
		}
		int tabuSearchMaxMiliseconds = 300000;//<=1min//300000;//<=5min
		int randomMaxMiliseconds = 1000;//1sec
		if(args.length == 7){//single alg.
			Algorithm a = null;
			char algorithm = args[6].toUpperCase().charAt(0);
//			String fileName = problemName;
			File file;
			AlgorithmResultGatherer gatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), algorithm, problem.getOptDistance());
			
			if(algorithm == Algorithm.TYPE_SIMPLE_HEURISTICS_NAME){
				a = new SimpleHeuristics(permutation); 
			}else if(algorithm == Algorithm.TYPE_GREEDY_NAME){
				a = new GreedyAlgorithm(permutation); 
			}else if(algorithm == Algorithm.TYPE_STEEPEST_NAME){
				a = new SteepestAlgorithm(permutation);
			}else if(algorithm == Algorithm.TYPE_RANDOM_NAME){
				a = new RandomAlgorithm(permutation);
			}else if(algorithm == Algorithm.TYPE_TABU_SEARCH_NAME){
				a = new TabuSearchAlgorithm(permutation);
			}else if(algorithm == Algorithm.TYPE_SIMULATED_ANNEALING_NAME){
				a = new SimulatedAnnealingAlgorithm(permutation);
			}else{
				System.out.println(Algorithm.ALGORITHM_TYPE_ERROR_MESSAGE);
				return;
			}
			file = new File(args[2]);//fileName);
			for(int i = 1; i <= multiRandomTimes; i++){			
				if(a instanceof RandomAlgorithm){					
					measuredTime = t.measure(a, 1, randomMaxMiliseconds);
				}else if(a instanceof TabuSearchAlgorithm){
					a.setMaxSteps(BigInteger.valueOf((long) Math.pow(permutation.getLength(), 3)));
					measuredTime = t.measure(a, 1, tabuSearchMaxMiliseconds);//5 min					
				}else if(a instanceof SimulatedAnnealingAlgorithm){
					a.setMaxSteps(BigInteger.valueOf((long) Math.pow(permutation.getLength(), 5)));
					measuredTime = t.measure(a, 1, tabuSearchMaxMiliseconds);//5 min					
				}else{
					measuredTime = t.measure(a, 1);					
				}
				if(algorithm == Algorithm.TYPE_GREEDY_NAME 
						|| algorithm == Algorithm.TYPE_STEEPEST_NAME 
						|| algorithm == Algorithm.TYPE_SIMULATED_ANNEALING_NAME 
						|| algorithm == Algorithm.TYPE_TABU_SEARCH_NAME){					
					gatherer.addResult(BigInteger.valueOf(measuredTime), a.getEndPermutation().countEvaluation(),
							a.getStepsMade(), a.getNodesChecked());
				}else{					
					gatherer.addResult(BigInteger.valueOf(measuredTime), a.getEndPermutation().countEvaluation());
				}
				a.setBeginPermutation(problem.generateRandomPermutation());
			}
			saveToFile(gatherer, file, append);
			permutationView(a.getEndPermutation(), a.getName()+": End Perm", file.getName().replace("result", "jpg"));	
			if(problem.hasOPTSolution())
				permutationView(problem.getSolution(), a.getName()+": OPT", problemName+"_OPT.jpg");	
		} else {//all alg.
			RandomAlgorithm rand = new RandomAlgorithm(permutation);
			SimpleHeuristics sh = new SimpleHeuristics(permutation);
			SteepestAlgorithm stp = new SteepestAlgorithm(permutation);
			GreedyAlgorithm grd = new GreedyAlgorithm(permutation);
			TabuSearchAlgorithm ts = new TabuSearchAlgorithm(permutation);
			SimulatedAnnealingAlgorithm sa = new SimulatedAnnealingAlgorithm(permutation);
			
			ts.setMaxSteps(BigInteger.valueOf((long) Math.pow(permutation.getLength(), 3)));
			sa.setMaxSteps(BigInteger.valueOf((long) Math.pow(permutation.getLength(), 5)));
			
			File randFile = new File(getProblemName2(rand, problemName));
			File shFile = new File(getProblemName2(sh, problemName));
			File stpFile = new File(getProblemName2(stp, problemName));
			File grdFile = new File(getProblemName2(grd, problemName));
			File tsFile = new File(getProblemName2(ts, problemName));
			File saFile = new File(getProblemName2(sa, problemName));
			
			AlgorithmResultGatherer randGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_RANDOM_NAME, problem.getOptDistance());
			AlgorithmResultGatherer shGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_SIMPLE_HEURISTICS_NAME, problem.getOptDistance());
			AlgorithmResultGatherer stpGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_STEEPEST_NAME, problem.getOptDistance());
			AlgorithmResultGatherer grdGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_GREEDY_NAME, problem.getOptDistance());
			AlgorithmResultGatherer tsGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_TABU_SEARCH_NAME, problem.getOptDistance());
			AlgorithmResultGatherer saGatherer = new AlgorithmResultGatherer(
					problemName, problem.getSize(), Algorithm.TYPE_SIMULATED_ANNEALING_NAME, problem.getOptDistance());
			for(int i = 1; i <= multiRandomTimes; i++){
				System.out.println("Uruchomienie nr: "+i);
				createResult(t.measure(rand, 1, 100), rand, randGatherer);
				createResult(t.measure(sh, 1), sh, shGatherer);
				createResult(t.measure(stp, 1), stp, stpGatherer);			
				createResult(t.measure(grd, 1), grd, grdGatherer);						
				createResult(t.measure(ts, 1, tabuSearchMaxMiliseconds), ts, tsGatherer);						
				createResult(t.measure(sa, 1, tabuSearchMaxMiliseconds), sa, saGatherer);						
				
				permutation = problem.generateRandomPermutation();
				rand.setBeginPermutation(permutation);
				sh.setBeginPermutation(permutation);
				stp.setBeginPermutation(permutation);
				grd.setBeginPermutation(permutation);
				ts.setBeginPermutation(permutation);
				sa.setBeginPermutation(permutation);
			}
			saveToFile(randGatherer, randFile, randFile.exists());
			saveToFile(shGatherer, shFile, shFile.exists());
			saveToFile(stpGatherer, stpFile, stpFile.exists());
			saveToFile(grdGatherer, grdFile, grdFile.exists());
			saveToFile(tsGatherer, tsFile, tsFile.exists());
			saveToFile(saGatherer, saFile, saFile.exists());

			if(problem.hasOPTSolution())
				permutationView(problem.getSolution(), ": OPT", problemName+"_OPT.jpg");	
			permutationView(rand.getEndPermutation(), rand.getName()+": End Perm", randFile.getName().replace("result", "jpg"));	
			permutationView(sh.getEndPermutation(), sh.getName()+": End Perm", shFile.getName().replace("result", "jpg"));	
			permutationView(stp.getEndPermutation(), stp.getName()+": End Perm", stpFile.getName().replace("result", "jpg"));	
			permutationView(grd.getEndPermutation(), grd.getName()+": End Perm", grdFile.getName().replace("result", "jpg"));	
			permutationView(ts.getEndPermutation(), ts.getName()+": End Perm", tsFile.getName().replace("result", "jpg"));	
			permutationView(sa.getEndPermutation(), sa.getName()+": End Perm", saFile.getName().replace("result", "jpg"));	
		}
	}	

	private static void createResult(long measuredTime, Algorithm a,
			AlgorithmResultGatherer gatherer) {
		if(a.getAlgorithmName() == Algorithm.TYPE_GREEDY_NAME 
				|| a.getAlgorithmName() == Algorithm.TYPE_STEEPEST_NAME 
				|| a.getAlgorithmName() == Algorithm.TYPE_TABU_SEARCH_NAME
				|| a.getAlgorithmName() == Algorithm.TYPE_SIMULATED_ANNEALING_NAME
				){					
			gatherer.addResult(BigInteger.valueOf(measuredTime), a.getEndPermutation().countEvaluation(),
					a.getStepsMade(), a.getNodesChecked());
		}else{					
			gatherer.addResult(BigInteger.valueOf(measuredTime), a.getEndPermutation().countEvaluation());
		}
	}

	private static String getProblemName2(Algorithm a, String problemName) {
		File f = new File(folderName);
		if(!f.exists()){
			f.mkdirs();
		}
		if(multiTSPTribe){
			return folderName+System.getProperty("file.separator")+a.getAlgorithmName()+".result";
		}else{
			return folderName+System.getProperty("file.separator")+getProblemName(a.getAlgorithmName(), problemName);
		}
	}

	private static String getProblemName(char algType, String problemName) {
		return problemName+"_"+algType+".result";
	}

	private static void saveToFile(AlgorithmResultGatherer gatherer, File file,
			boolean append) {
		try {
			BufferedWriter output = new BufferedWriter(new FileWriter(file, append));
			try {
				output.write(gatherer.printResult(spacer, append));
			} finally {
				output.close();
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public static String getProblemFileName(String path) {
		int index = path.lastIndexOf(System.getProperty("file.separator"));
		if(index != -1) //path like: 'd:/something/folder/file.tsp'
			return path.substring(index);
		else //path is filename like: 'file.tsp'
			return path;
	}

	private static void permutationView(final Permutation permutation, final String title, String imageName) {
		if(savePicture){
			CreateImage ci = new CreateImage(permutation, "file");
			try {
				ci.createImage(imageName);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if(showWindow){
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					new TravelPathWindow(permutation, title+". SUM = "+permutation.countEvaluation()).setVisible(true);
				}
			});	
		}
	}
}
