package tsp.model;

import java.util.Random;

public class Permutation {
	int[] list;
	int actualIndex = 0;//numer miasta z listy, który wyznacza początek
	Double lastEvaluation = 0.0;
	Problem problem;
	
	public Permutation(int size, Problem problem){
		list = new int[size];
		for(int i = 0; i < size; i++){
			list[i] = i;
		}
		this.problem = problem;
	}
	
	private Permutation(int[] lista, int actualIndex,
			Problem problem, Double lastEvaluation) {
		super();
		this.list = new int[lista.length];
		for(int i = 0; i < lista.length; i++){
			this.list[i] = lista[i];
		}
		this.actualIndex = actualIndex;
		this.problem = problem;
		this.lastEvaluation = new Double(lastEvaluation);
	}

	public Permutation(int[] optimalSolution, Problem problem) {
		super();
		this.list = new int[optimalSolution.length];
		for(int i = 0; i < optimalSolution.length; i++){
			this.list[i] = optimalSolution[i]; //"-1" is because optimal solution has scale: 1..n, while permutation has 0..n-1
		}
		this.problem = problem;
	}
	
	/*
	 * Returns number in sequence including offset and different begin index (actualIndex)
	 */
	private int num(int offset){
		return (offset+actualIndex)%list.length;
	}

	public Double swapCitiesEvaluation(int k, int l){
		swap(k, l);
		Double result = countEvaluation(); 
		swap(k, l);
		return result;
	}

	public void randomSwapAll() {
		Random generator = new Random(System.nanoTime());
		int r = generator.nextInt();
		int srodkowy = list.length / 2;
		for(int i = list.length; i > 0; i--){
			r = generator.nextInt(i);
			swap(srodkowy, r);
		}
	}

	public Double countEvaluation() {
		if(list != null && list.length > 0){
			Double evaluation = 0.0;
			for(int i = 0; i < list.length-1; i++){
				evaluation += problem.distanceTable[list[i]][list[i+1]];
			}
			evaluation += problem.distanceTable[list[0]][list[list.length-1]];
			lastEvaluation = evaluation;
			return evaluation;
		}else{
			return 0.0d;
		}
	}

	public Double randomSwapEvaluation() {
		Random generator = new Random(System.nanoTime());
		int r = generator.nextInt(list.length);
		swap(r, (r + generator.nextInt(list.length))%list.length);
		return countEvaluation();
	}

	public void swap(int two, int one) {
		int temp = list[num(one)];
		list[num(one)] = list[num(two)];
		list[num(two)] = temp;
	}

	public String print() {
		StringBuilder sb = new StringBuilder(""); 
		for(int i = 0; i < list.length; i++){
//			System.out.print(lista[i]+" ");
			sb.append(list[i]+" ");
		}
//		System.out.println();
		return sb.toString();
	}

	public Permutation cloning() {
		return new Permutation(list, actualIndex, problem, lastEvaluation);
	}

	public int getLength() {
		return list.length;
	}

	public Double getLastEvaluation() {
		return lastEvaluation;
	}
	
	public Double getDistance(int k, int l){
		return problem.distanceTable[list[k]][list[l]];
	}

	public int[] getList() {
		return list;
	}

	public Problem getProblem() {
		return problem;
	}

	public void setActualIndex(int actualIndex) {
		this.actualIndex = actualIndex;
	}
}
