package tsp.model;

import java.awt.geom.Point2D;
import java.util.List;

public class Problem {
	Double[][] distanceTable;
	int size;
	List<Point2D> list;
	Permutation solution;
	Double optDistance;
	

	public Problem(Double[][] distanceTable2, List<Point2D> list, int[] optimalSolution) {
		this.list = list;
		distanceTable = distanceTable2;
		size = distanceTable.length;
		if(optimalSolution != null && optimalSolution.length == size){
			this.solution = new Permutation(optimalSolution, this);
			this.optDistance = solution.countEvaluation();
		}else{
			this.solution = null;
			this.optDistance = -1d;
		}
	}

	public Permutation generateRandomPermutation() {
		Permutation p = new Permutation(size, this);
		p.randomSwapAll();
		return p;
	}

	public void printDistanceTable(){
		for(int i = 0; i < distanceTable.length; i++){
			for(int j = 0; j < distanceTable.length; j++){
				System.out.print(distanceTable[i][j]);			
			}
			System.out.println("");
		}
	}

	public int getSize() {
		return size;
	}

	public List<Point2D> getList() {
		return list;
	}

	public boolean hasOPTSolution() {
		return solution != null;
	}

	public Permutation getSolution() {
		return solution;
	}

	public Double getOptDistance() {
		return optDistance;
	}
}
