package tsp.readInput;

//import java.awt.Point;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import tsp.model.Problem;
import tsp.readInput.exception.WrongProblemDefinisionException;

public class ProblemReader {

	static boolean readingData = false;
	static List<Point2D> list = new ArrayList<Point2D>();
	static int[] optimalSolution = null;
	
	public ProblemReader(){
		list = new ArrayList<Point2D>();
		optimalSolution = null;
	}

	public Problem read(String filename) throws WrongProblemDefinisionException {
		File testFile = new File(filename);//"ts225.tsp");
		createPoints(testFile);
		File resolutionFile = new File(filename.replaceFirst(".tsp", ".opt.tour"));
		if(resolutionFile.exists()){
			optimalSolution = readSolution(resolutionFile, list.size());
		}
		return createProblem();
	}

	private int[] readSolution(File resolutionFile, int length) {
		int[] result = new int[length];
		readingData = false;
		int i = 0;
		try {
			// use buffering, reading one line at a time
			BufferedReader input = new BufferedReader(new FileReader(resolutionFile));
			try {
				String line = null; 
				while ((line = input.readLine()) != null) {
					Pattern pattern = Pattern.compile("TOUR_SECTION");
					Matcher matcher = pattern.matcher(line);
					if (matcher.find()) {
						readingData = true;
						continue;
					}
					Pattern pattern2 = Pattern.compile("-1");
					Matcher matcher2 = pattern2.matcher(line);
					if (matcher2.find() || i >=length) {
						readingData = false;
					}
					if (readingData) {
						StringTokenizer st = new StringTokenizer(line);
						while (st.hasMoreTokens()) {
							int temp = Integer.parseInt(st.nextToken());
							result[i] = temp-1;//-1 because we use scale from 0..n-1, and not 1..n
							i++;
						}
					}
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		return result;
	}

	static private void createPoints(File aFile) throws WrongProblemDefinisionException {
		try {
			// use buffering, reading one line at a time
			BufferedReader input = new BufferedReader(new FileReader(aFile));
			try {
				String line = null; 
				while ((line = input.readLine()) != null) {

					Pattern pattern = Pattern.compile("NODE_COORD_SECTION");
					Matcher matcher = pattern.matcher(line);
					if (matcher.find()) {
						readingData = true;
						continue;
					}
					Pattern pattern2 = Pattern.compile("EOF");
					Matcher matcher2 = pattern2.matcher(line);
					if (matcher2.find()) {
						readingData = false;
					}
					if (readingData) {
						StringTokenizer st = new StringTokenizer(line);
						while (st.hasMoreTokens()) {
							st.nextToken();
							Point2D temp = new Point2D.Double(
									Double.parseDouble(st.nextToken()), 
									Double.parseDouble(st.nextToken())
									/*, Integer.parseInt(st.nextToken())*/);
							list.add(temp);
						}
					}
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}finally{
			readingData = false;
			if(list == null || list.size() == 0){
				throw new WrongProblemDefinisionException();
			}
		}
	}

	private Problem createProblem() {
		Double[][] distanceTable = new Double[list.size()][list.size()];
		int i = 0, j = 0;
		for (Point2D t : list) {
			for (Point2D t2 : list) {
				distanceTable[i][j] = distance(t, t2);
				j++;
			}
			i++;
			j = 0;
		}
		return new Problem(distanceTable, list, optimalSolution);
	}

	private Double distance(Point2D t, Point2D t2) {
		Double result;
		result = Math.sqrt(Math.pow(t.getX() - t2.getX(), 2) + Math.pow(t.getY() - t2.getY(), 2));
		return result;
	}
}
