package tsp.utils;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/** Gathers results for one algorithm and one problem instance.
 * e.g. If you start algorithm A on problem P 100 times
 *  - this class can store all the results of these lauchings 
 * */
public class AlgorithmResultGatherer {
	int instanceSize;
	char algorithm;
	String instanceName;
	Double optimalDistanceSum;
	List<BigInteger> timeResults = new ArrayList<BigInteger>();
	List<Double> distanceSumResults = new ArrayList<Double>();
	List<BigInteger> stepsMadeResults = new ArrayList<BigInteger>();
	List<BigInteger> nodesVisitedResults = new ArrayList<BigInteger>();
	
	public AlgorithmResultGatherer(String instanceName, int instanceSize, char algorithm, Double optDistanceSum){
		this.instanceName = instanceName;
		this.instanceSize = instanceSize;
		this.algorithm = algorithm;
		this.optimalDistanceSum = optDistanceSum;
	}
	
	public void addResult(BigInteger time, Double distanceSum){
		timeResults.add(time);
		distanceSumResults.add(distanceSum);
	}
	
	public void addResult(BigInteger time, Double distanceSum, BigInteger stepsMade, BigInteger nodesVisited){
		timeResults.add(time);
		distanceSumResults.add(distanceSum);
		stepsMadeResults.add(stepsMade);
		nodesVisitedResults.add(nodesVisited);
	}

	private String getStatisticsBI(List<BigInteger> list, char spacer){
		StringBuilder result = new StringBuilder();
		if(list.size() > 0){
			int minElement = 0;
			int maxElement = 0;
			BigInteger medium = BigInteger.ZERO;
			BigInteger variationSum = BigInteger.ZERO;
			double variation;
			
			for (BigInteger val : list) {
				medium = medium.add(val);
			}
			medium = medium.divide(BigInteger.valueOf(list.size()));
			
			for (int i = 0; i < list.size(); i++) {
				if(list.get(i).compareTo(list.get(maxElement)) > 0){
					maxElement = i;
				}
				if(list.get(i).compareTo(list.get(minElement)) < 0){
					minElement = i;
				}
				variationSum = variationSum.add(list.get(i).subtract(medium).pow(2));
			}
			variation = Math.sqrt(variationSum.divide(BigInteger.valueOf(list.size())).doubleValue());
			result.append(medium).append(spacer)
				.append(variation).append(spacer)
				.append(list.get(maxElement)).append(spacer)
				.append(list.get(minElement));
			
		}else{		
			result.append("0").append(spacer)
				.append("0.0").append(spacer)
				.append("0").append(spacer)
				.append("0");
		}
		return result.toString();
	}

	private String getStatisticsD(List<Double> list, char spacer){
		StringBuilder result = new StringBuilder();
		if(list.size() > 0){
			int minElement = 0;
			int maxElement = 0;
			Double medium = 0d;
			Double variationSum = 0d;
			Double variation;
			
			for (Double val : list) {
				medium += val;
			}
			medium = medium/Double.valueOf(list.size());
			
			for (int i = 1; i < list.size(); i++) {
				if(list.get(i).compareTo(list.get(maxElement)) > 0){
					maxElement = i;
				}
				if(list.get(i).compareTo(list.get(minElement)) < 0){
					minElement = i;
				}
				variationSum += Math.pow(list.get(i)- medium, 2);
			}
			variation = Math.sqrt(variationSum/Double.valueOf(list.size()));
			result.append(medium).append(spacer)
				.append(variation).append(spacer)
				.append(list.get(maxElement)).append(spacer)
				.append(list.get(minElement));
			
		}else{		
			result.append("0").append(spacer)
				.append("0.0").append(spacer)
				.append("0").append(spacer)
				.append("0");
		}
		return result.toString();
	}

	public String printResult(char spacer, boolean append) {
		StringBuilder result = new StringBuilder();
		if(!append)
			result.append("#").append(algorithm).append("\n");
		result.append(algorithm).append(spacer)
			.append(instanceName).append(spacer)
			.append(instanceSize).append(spacer)
			.append(optimalDistanceSum).append(spacer)
			.append(getStatisticsBI(timeResults, spacer)).append(spacer)
			.append(getStatisticsD(distanceSumResults, spacer)).append(spacer)
			.append(getStatisticsBI(stepsMadeResults, spacer)).append(spacer)
			.append(getStatisticsBI(nodesVisitedResults, spacer))
			.append("\n");
		return result.toString();
	}
}
