package tsp.utils;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;

import tsp.model.Permutation;

public class CreateImage {

	private static final long serialVersionUID = -8525367129467532828L;
	Permutation permutation;
	double minX = Integer.MAX_VALUE;
	double maxX = Integer.MIN_VALUE;
	double minY = Integer.MAX_VALUE;
	double maxY = Integer.MIN_VALUE;
	List<Point2D> points;
	int pointsList[];
	int windowSize = 2400;
	int border = windowSize/50;
//	private String name;
	
	public CreateImage(Permutation permutation, String name){
//    	this.name = name;//write it on the top?
    	this.permutation = permutation.cloning();
    	this.pointsList = permutation.getList();
    	this.points = permutation.getProblem().getList();
    	for(Point2D cityPoint : permutation.getProblem().getList()){
    		if(minX > cityPoint.getX()){
    			minX = cityPoint.getX();
    		}
    		if(maxX < cityPoint.getX()){
    			maxX = cityPoint.getX();
    		}
    		if(minY > cityPoint.getY()){
    			minY = cityPoint.getY();
    		}
    		if(maxY < cityPoint.getY()){
    			maxY = cityPoint.getY();
    		}
    	}
    	scalePoints();
	}

	public BufferedImage createImage(String fileName) throws IOException {        
		BufferedImage image = new BufferedImage(windowSize+2*border, windowSize+2*border,
				BufferedImage.TYPE_INT_RGB);

		Graphics2D g = (Graphics2D) image.getGraphics();
		g.setBackground(Color.white);
		g.clearRect(0, 0, windowSize+2*border, windowSize+2*border);
		paint(g);
//		g.dispose();
		CreateImage.writeImage(image, fileName);
		return image;
	}
    
    private void scalePoints() {
		for(int i = 0; i < pointsList.length; i++){
			((Point2D.Double)points.get(i)).x = (points.get(i).getX() - minX)*windowSize/(maxX - minX)+border;
			((Point2D.Double)points.get(i)).y = (points.get(i).getY() - minY)*windowSize/(maxY - minY)+border;
		}
	}

	/**
     * This is the method where the line is drawn.
     *
     * @param g The graphics object
     */
    public void paint(Graphics2D g2) {
//        Graphics2D g2 = (Graphics2D) g;
        g2.setBackground(Color.white);
        g2.setColor(Color.black);
        for(int i = 0; i < permutation.getLength()-1; i++){
        	g2.draw(new Line2D.Double(points.get(pointsList[i]), points.get(pointsList[i+1])));
        }
        g2.draw(new Line2D.Double(points.get(pointsList[0]), points.get(pointsList[permutation.getLength()-1])));
        g2.setColor(Color.red);
        Ellipse2D.Double ellipse;
        double xsize = windowSize/100;
        for(Point2D p : points){
        	ellipse = new Ellipse2D.Double(p.getX()-xsize/2, p.getY()-xsize/2, xsize, xsize);
        	g2.draw(ellipse);
        	g2.fill(ellipse);
        }
        g2.setColor(Color.black);        
    }

	public static void writeImage(BufferedImage image, String fileName)
			throws IOException {
		if (fileName == null)
			return;
//		fileName = System.currentTimeMillis()+".jpg";

		int offset = fileName.lastIndexOf(".");
		String type = offset == -1 ? "jpg" : fileName.substring(offset + 1);
		System.out.println(fileName+" image created");
		ImageIO.write(image, type, new File(fileName));
	}
}
