package tsp.window;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.List;

import javax.swing.GroupLayout;

import tsp.model.Permutation;


public class TravelPathWindow extends javax.swing.JFrame {
    
	private static final long serialVersionUID = -8525367129467532828L;
	Permutation permutation;
	double minX = Integer.MAX_VALUE;
	double maxX = Integer.MIN_VALUE;
	double minY = Integer.MAX_VALUE;
	double maxY = Integer.MIN_VALUE;
	List<Point2D> points;
	int pointsList[];
	int windowSize = 550;
	int border = 10;
	private String name;

	/**
	 * @param permutation 
     */
    public TravelPathWindow(Permutation permutation, String name) {
    	this.name = name;
    	this.permutation = permutation.cloning();
    	this.pointsList = permutation.getList();
    	this.points = permutation.getProblem().getList();
    	for(Point2D cityPoint : permutation.getProblem().getList()){
    		if(minX > cityPoint.getX()){
    			minX = cityPoint.getX();
    		}
    		if(maxX < cityPoint.getX()){
    			maxX = cityPoint.getX();
    		}
    		if(minY > cityPoint.getY()){
    			minY = cityPoint.getY();
    		}
    		if(maxY < cityPoint.getY()){
    			maxY = cityPoint.getY();
    		}
    	}
//    	System.out.println("X : <"+minX+", "+maxX+">");
//    	System.out.println("Y : <"+minY+", "+maxY+">");
    	scalePoints();
        initComponents();
    }
    
    private void scalePoints() {
		for(int i = 0; i < pointsList.length; i++){
			((Point2D.Double)points.get(i)).x = (points.get(i).getX() - minX)*windowSize/(maxX - minX)+border;
			((Point2D.Double)points.get(i)).y = (points.get(i).getY() - minY)*windowSize/(maxY - minY)+border*3;
		}
	}

	/**
     * This is the method where the line is drawn.
     *
     * @param g The graphics object
     */
    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setBackground(Color.white);
        for(int i = 0; i < permutation.getLength()-1; i++){
        	g2.draw(new Line2D.Double(points.get(pointsList[i]), points.get(pointsList[i+1])));
//        	drawLineBetweenPoints(g2, points.get(pointsList[i]), points.get(pointsList[i+1]));
        }
        g2.draw(new Line2D.Double(points.get(pointsList[0]), points.get(pointsList[permutation.getLength()-1])));
//        drawLineBetweenPoints(g2, points.get(pointsList[0]), points.get(pointsList[permutation.getLength()-1]));
        Ellipse2D.Double ellipse;
        double xsize = windowSize/100;
        g2.setColor(Color.red);        
        for(Point2D p : points){
        	ellipse = new Ellipse2D.Double(p.getX()-xsize/2, p.getY()-xsize/2, xsize, xsize);
        	g2.draw(ellipse);
        	g2.fill(ellipse);
        }
        g2.setColor(Color.black);        
    }

	private void initComponents() {
		this.setTitle(name);
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        GroupLayout layout = new GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, windowSize+2*border, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, windowSize+2*border, Short.MAX_VALUE)
        );
        pack();
    }    
}